<?php

# Check optimal database configuration

function get_mysql_suggest_count($sugs,$min_level,$max_level) {
  $cnt=0;
  if (is_array($sugs))
    reset($sugs);
  foreach ($sugs as $s)
    if (is_array($s) && $s[0]>=$min_level && $s[0]<=$max_level)
      $cnt++;
  return ' '.$cnt;
}


function get_mysql_suggest() {
  $cnt=0;
  $sugs=array();

  $meminfo=get_meminfo();
  $mysqlinfo=get_mysqlinfo();
  $vars=$mysqlinfo['vars'];
  $stat=$mysqlinfo['stat'];
  $cals=get_calc($meminfo,$vars,$stat);


# log-bin
  if (isset($vars['log_bin']) && strtoupper($vars['log_bin'])!='OFF')
    $sugs[$cnt]=array(3,
	x('Binary logging in use.'),
	x("All SQL queries are logged to: %s.",$vars['log_bin']),
	x("Disable or comment log-bin option. Binary logging is a huge power consumer.")
	);
  else
    $sugs[$cnt]=array(0,
	x('Binary logging is disabled.'),
	x("All SQL queries are not logged in to the binary file."),
	""
	);
  $cnt++;


# Disk space
  if (defined('PRG_DBHOST') && (PRG_DBHOST=='' || PRG_DBHOST=='localhost') && isset($vars['datadir']) && function_exists('disk_total_space') && is_dir($vars['datadir'])) {
    $dfs=disk_free_space($vars['datadir']);
    $dts=disk_total_space($vars['datadir']);
    $df=round(($dfs/$dts)*100);
    if ($df<10 || $dfs<500*1024*1024)
      $sugs[$cnt]=array(3,
	x('Low disk space for database tables.'),
	x("Current disk free space is %sB/%sB (%s%%).",number2human($dfs,"up"),number2human($dts,"up"),$df),
	x("Add a new disk to the disk array (RAID) or decrease data history (Options->Collectors->Edit).")
	);
    else
      $sugs[$cnt]=array(0,
	x('Sufficient disk free space for data.'),
	x("Current disk free space is %sB/%sB (%s%%).",number2human($dfs,"up"),number2human($dts,"up"),$df),
	""
	);
    $cnt++;
  }


  if (!isset($meminfo['MemTotal']) || !$meminfo['MemTotal']) {
      $sugs[$cnt]=array(1,
	x('Could not get a physical memory size.'),
	"",
	x("Check if /proc/meminfo file is accessible or copy file to the /usr/share/netflow/web/meminfo.")
	);
    $cnt++;
  }  


# Memory check - Max possible
  if (isset($meminfo['MemTotal']) && $meminfo['MemTotal']) {
    if ($cals['pct_virtual_memory'] < 90)  // virtual = physical + swap
      $sugs[$cnt]=array(0,
	x('Optimal max-possible physical memory usage.'),
	x("%sB - %s %% of installed %sB RAM.",number2human($cals['total_possible_used_memory'],"up"),floor($cals['pct_physical_memory']),number2human($meminfo['MemTotal'],"up")),
	""
	);
    else
      $sugs[$cnt]=array(1,
	x('High max-possible physical memory usage.'),
	x("%sB - %s %% of installed %sB RAM.",number2human($cals['total_possible_used_memory'],"up"),floor($cals['pct_physical_memory']),number2human($meminfo['MemTotal'],"up")),
	x("Install an additional memory or decrease key_buffer, max_heap_table, tmp_table_size, query_cache, read_buffer, read_rnd_buffer, sort_buffer, max_connections.")
	);
    $cnt++;

# Memory check - Used
    if ($cals['pct_server_physical_memory'] < 95 )
      $sugs[$cnt]=array(0,
	x('Optimal physical memory peak usage.'),
	x("%sB - %s %% of installed %sB RAM.",number2human($cals['server_buffers'],"up"),floor($cals['pct_server_physical_memory']),number2human($meminfo['MemTotal'],"up")),
	""
	);
    else
      $sugs[$cnt]=array(3,
	x('High physical memory peak usage.'),
	x("%sB - %s %% of installed %sB RAM.",number2human($cals['server_buffers'],"up"),floor($cals['pct_server_physical_memory']),number2human($meminfo['MemTotal'],"up")),
	x("Install an additional memory or decrease key_buffer, max_heap_table, tmp_table_size, query_cache.")
	);
    $cnt++;
  }


# Max_allowed_packet
  if (isset($vars['max_allowed_packet']) && $vars['max_allowed_packet']< 1024*1024)
    $sugs[$cnt]=array(3,
	x('Small max_allowed_packet option.'),
	x("Max_allowed_packet = %sB.",number2human($vars['max_allowed_packet'],"up")),
	x("NetFlow Collector requires 1MB at minimum.")
	);
  else
    $sugs[$cnt]=array(0,
	x('Optimal max_allowed_packet option.'),
	x("Max_allowed_packet = %sB.",number2human($vars['max_allowed_packet'],"up")),
	""
	);
  $cnt++;


# Connections
  if ($cals['pct_connections_used'] >= 85)
    $sugs[$cnt]=array(3,
	x('High connection usage.'),
	x("%s%% (%s/%s) connections.",round($cals['pct_connections_used']),$stat['Max_used_connections'],$vars['max_connections']),
	x("Increase max_connections to %s or more.",ceil($stat['Max_used_connections']/0.85))
	);
  else
  if ($cals['pct_connections_used'] < 30)
    $sugs[$cnt]=array(1,
	x('Too low connection usage.'),
	x("%s%% (%s/%s) connections.",round($cals['pct_connections_used']),$stat['Max_used_connections'],$vars['max_connections']),
	x("Decrease max_connections parameter. The max_connections can waste a memory. We recommend to set max_connections parameter to: %s.",max(10,ceil($stat['Max_used_connections']*2.5)))
	);
  else
    $sugs[$cnt]=array(0,
	x('Optimal connection usage.'),
	x("%s%% (%s/%s) connections.",round($cals['pct_connections_used']),$stat['Max_used_connections'],$vars['max_connections']),
	""
	);
  $cnt++;



# Key buffer
    if ($vars['version_major'] == 3) {
      $sugs[$cnt]=array(1,
	x("Upgrade to version 4.0.1+ to utilize key_buffers."),
	x("Key buffers can dramatically increase overall performance."),
	""
	);
      $cnt++;
    }

    if ($stat['Uptime'] > 1800 && isset($meminfo['MemTotal']) && $meminfo['MemTotal'] && $vars['version_major'] != 3) {
    if ($vars['key_buffer_size'] < ($meminfo['MemTotal']/8))
      $sugs[$cnt]=array(2,
	x("Small key buffer size."),
	x("%sB - %s %% of installed %sB RAM - %s%% keys from memory.",number2human($vars['key_buffer_size'],"up"),round($cals['pct_key_buffer_size']),number2human($meminfo['MemTotal'],"up"),round($cals['pct_keys_from_mem'])),
	x("Increase key_buffer_size to %sB as a starting value.",number2human($meminfo['MemTotal']/4,"up"))
	);
    elseif ($vars['key_buffer_size'] > ($meminfo['MemTotal']/2))
      $sugs[$cnt]=array(3,
	x("Too high key buffer size."),
	x("%sB - %s %% of installed %sB RAM - %s%% keys from memory.",number2human($vars['key_buffer_size'],"up"),round($cals['pct_key_buffer_size']),number2human($meminfo['MemTotal'],"up"),round($cals['pct_keys_from_mem'])),
	x("Decrease key_buffer_size to %sB.",number2human($meminfo['MemTotal']/2,"up"))
	);
    elseif ($cals['pct_keys_from_mem'] < 85 && $meminfo['MemTotal'] <= 1024*1024*1024)
      $sugs[$cnt]=array(2,
	x("Small key buffer size."),
	x("%sB - %s %% of installed %sB RAM - %s%% keys from memory.",number2human($vars['key_buffer_size'],"up"),round($cals['pct_key_buffer_size']),number2human($meminfo['MemTotal'],"up"),round($cals['pct_keys_from_mem'])),
	x("Little increase key_buffer_size (max. to %sB) or increase size of physical memory = recommended option. The optimal value is >85% keys from memory.",number2human($meminfo['MemTotal']/2,"up"))
	);
    elseif ($cals['pct_keys_from_mem'] < 85 && $meminfo['MemTotal'] > 1024*1024*1024)
      $sugs[$cnt]=array(2,
	x("Small key buffer size."),
	x("%sB - %s %% of installed %sB RAM - %s%% keys from memory.",number2human($vars['key_buffer_size'],"up"),round($cals['pct_key_buffer_size']),number2human($meminfo['MemTotal'],"up"),round($cals['pct_keys_from_mem'])),
	x("Little increase key_buffer_size (max. to %sB) or increase size of physical memory. The optimal value is >85% keys from memory.",number2human($meminfo['MemTotal']/2,"up"))
	);
    else
      $sugs[$cnt]=array(0,
	x("Optimal key buffer size."),
	x("%sB - %s %% of installed %sB RAM - %s%% keys from memory.",number2human($vars['key_buffer_size'],"up"),round($cals['pct_key_buffer_size']),number2human($meminfo['MemTotal'],"up"),round($cals['pct_keys_from_mem'])),
	""
	);
    $cnt++;
    }

# Query cache
    if ($vars['version_major'] == 3 || !isset($vars['key_cache_block_size']))
      $sugs[$cnt]=array(1,
	x("Upgrade to version 4.0.1+ to utilize query caching."),
	x("Query caching can dramatically increase overall performance"),
	""
	);
    elseif ($vars['query_cache_size'] < 1)
      $sugs[$cnt]=array(2,
	x("Query caching is disabled."),
	x("Query_cache_size = %sB - efficiency 0%%.",number2human($vars['query_cache_size'],"up")),
	x("Set query_cache_size >= 8MB as a starting value.")
	);
    elseif ($stat['Com_select'] > 100 && $cals['query_cache_efficiency'] < 20)
      $sugs[$cnt]=array(1,
	x("Query caching has a small efficiency."),
	x("Query_cache_size = %sB - efficiency %s%%.",number2human($vars['query_cache_size'],"up"),round($cals['query_cache_efficiency'])),
	x("Increase query_cache_size.")
	);
    else
      $sugs[$cnt]=array(0,
	x("Optimal query cache size."),
	x("Query_cache_size = %sB - efficiency %s%%.",number2human($vars['query_cache_size'],"up"),round($cals['query_cache_efficiency'])),
	""
	);
    $cnt++;


# Sorting
    if ($cals['total_sorts'] && $cals['pct_temp_sort_table'] > 20)
      $sugs[$cnt]=array(2,
	x("The sorts are requiring many temporary tables."),
	x("Sort_buffer = %sB, Read_rnd_buffer=%sB, %s%% temporary sort tables.",number2human($vars['sort_buffer_size'],"up"),number2human($vars['read_rnd_buffer_size'],"up"),round($cals['pct_temp_sort_table'])),
	x("Increase sort_buffer_size and read_rnd_buffer_size if possible.")
	);
    else
      $sugs[$cnt]=array(0,
	x("Optimal sort and read_rnd buffers."),
	x("Sort_buffer = %sB, Read_rnd_buffer=%sB, %s%% temporary sort tables.",number2human($vars['sort_buffer_size'],"up"),number2human($vars['read_rnd_buffer_size'],"up"),round($cals['pct_temp_sort_table'])),
	""
	);
    $cnt++;


# Temporary tables
    if (isset($meminfo['MemTotal']) && $meminfo['MemTotal']) {
      if ($stat['Created_tmp_tables'] && $cals['max_tmp_table_size'] <= 10*1024*1024)
        $sugs[$cnt]=array(2,
	x("Temporary tables are created on disk."),
	x("Tmp_table_size = %sB, Max_heap_table_size = %sB, %s%% temporary disk tables.",number2human($vars['tmp_table_size'],"up"),number2human($vars['max_heap_table_size'],"up"),round($cals['pct_temp_disk'])),
	x("Increase tmp_table_size and max_heap_table_size.")
	);
      elseif ($stat['Created_tmp_tables'] && $cals['pct_temp_disk'] > 25 && $cals['max_tmp_table_size']<$meminfo['MemTotal']/4)
        $sugs[$cnt]=array(2,
	x("Temporary tables are created on disk."),
	x("Tmp_table_size = %sB, Max_heap_table_size = %sB, %s%% temporary disk tables.",number2human($vars['tmp_table_size'],"up"),number2human($vars['max_heap_table_size'],"up"),round($cals['pct_temp_disk'])),
	x("Increase tmp_table_size and max_heap_table_size.")
	);
      elseif ($vars['tmp_table_size']<$meminfo['MemTotal']/16)
        $sugs[$cnt]=array(1,
	x("Small size of temporary tables."),
	x("Tmp_table_size = %sB, Max_heap_table_size = %sB, %s%% temporary disk tables.",number2human($vars['tmp_table_size'],"up"),number2human($vars['max_heap_table_size'],"up"),round($cals['pct_temp_disk'])),
	x("Increase tmp_table_size and max_heap_table_size to %sB at minimum.",number2human($meminfo['MemTotal']/16,"up"))
	);
      elseif ($vars['tmp_table_size']>floor(($meminfo['MemTotal']-$vars['key_buffer_size'])*0.9))
        $sugs[$cnt]=array(2,
	x("Too large size of temporary tables."),
	x("Tmp_table_size = %sB, Max_heap_table_size = %sB, %s%% temporary disk tables.",number2human($vars['tmp_table_size'],"up"),number2human($vars['max_heap_table_size'],"up"),round($cals['pct_temp_disk'])),
	x("Decrease tmp_table_size and max_heap_table_size to %sB at maximum.",number2human(floor(($meminfo['MemTotal']-$vars['key_buffer_size'])*0.8),"up"))
	);
      else
        $sugs[$cnt]=array(0,
	x("Optimal tmp_table_size and max_heap_table_size."),
	x("Tmp_table_size = %sB, Max_heap_table_size = %sB, %s%% temporary disk tables.",number2human($vars['tmp_table_size'],"up"),number2human($vars['max_heap_table_size'],"up"),round($cals['pct_temp_disk'])),
	""
	);
      $cnt++;
   }

# Thread cache
    if ($vars['thread_cache_size'] == 0)
      $sugs[$cnt]=array(2,
	x("Thread cache is disabled."),
	x("Thread_cache_size = %s, Thread_cache_hit_rate = %s%%.",$vars['thread_cache_size'],round($cals['thread_cache_hit_rate'])),
	x("Set thread_cache_size to 4 as a starting value.")
	);
    elseif ($cals['thread_cache_hit_rate'] <= 50)
      $sugs[$cnt]=array(2,
	x("Thread cache is too small."),
	x("Thread_cache_size = %s, Thread_cache_hit_rate = %s%%.",$vars['thread_cache_size'],round($cals['thread_cache_hit_rate'])),
	x("Increase thread_cache_size.")
	);
    else
      $sugs[$cnt]=array(0,
	x("Optimal thread_cache_size."),
	x("Thread_cache_size = %s, Thread_cache_hit_rate = %s%%.",$vars['thread_cache_size'],round($cals['thread_cache_hit_rate'])),
	""
	);
    $cnt++;

  return $sugs;
}



function get_calc($meminfo,$vars,$stat) {
  $cals=array();
  if ($vars['version_major']>3)
    $cals['thread_buffers']=$vars['read_buffer_size'] + $vars['read_rnd_buffer_size'] + $vars['sort_buffer_size'] + $vars['thread_stack'] + $vars['join_buffer_size'];
  else
    $cals['thread_buffers']=$vars['record_buffer'] + $vars['record_rnd_buffer'] + $vars['sort_buffer'] + $vars['thread_stack'] + $vars['join_buffer_size'];

  $cals['total_thread_buffers'] = $cals['thread_buffers'] * $vars['max_connections'];
  $cals['max_total_thread_buffers'] = $cals['thread_buffers'] * $stat['Max_used_connections'];

  $cals['max_tmp_table_size'] = min($vars['tmp_table_size'],$vars['max_heap_table_size']);
  $cals['server_buffers'] = $vars['key_buffer_size'] + $cals['max_tmp_table_size'] + $vars['query_cache_size'];

  $cals['max_used_memory'] = $cals['server_buffers'] + $cals['max_total_thread_buffers'];
  $cals['total_possible_used_memory'] = $cals['server_buffers'] + $cals['total_thread_buffers'];
  $cals['pct_physical_memory'] = ($cals['total_possible_used_memory'] * 100) / $meminfo['MemTotal'];
  $cals['pct_virtual_memory'] = ($cals['total_possible_used_memory'] * 100) / ($meminfo['MemTotal']+$meminfo['SwapTotal']);
  $cals['pct_used_physical_memory'] = ($cals['max_used_memory'] * 100) / $meminfo['MemTotal'];
  $cals['pct_server_physical_memory'] = ($cals['server_buffers'] * 100) / $meminfo['MemTotal'];
  
  $cals['pct_connections_used'] = min(100,($stat['Max_used_connections'] * 100 ) / $vars['max_connections']);

  # Key buffers
  if ($vars['version_major'] > 3 && !($vars['version_major'] == 4 && $vars['version_minor'] == 0)) {
    $cals['pct_key_buffer_used'] = (1 - (($stat['Key_blocks_unused'] * $vars['key_cache_block_size']) / $vars['key_buffer_size'])) * 100;
    $cals['pct_key_buffer_size'] = ($vars['key_buffer_size'] * 100 ) / $meminfo['MemTotal'];
  }

  if ($stat['Key_read_requests'] > 0)
    $cals['pct_keys_from_mem'] = (100 - (($stat['Key_reads'] / $stat['Key_read_requests']) * 100));

  # Query cache
  if ($vars['version_major'] > 3)
    $cals['query_cache_efficiency'] = ($stat['Qcache_hits'] / ($stat['Com_select'] + $stat['Qcache_hits'])) * 100;

  if ($vars['query_cache_size'])
    $cals['pct_query_cache_used'] = 100 - ($stat['Qcache_free_memory'] / $vars['query_cache_size']) * 100;

  # Sorting
  $cals['total_sorts'] = $stat['Sort_scan'] + $stat['Sort_range'];
  if ($cals['total_sorts'] > 0)
    $cals['pct_temp_sort_table'] = ($stat['Sort_merge_passes'] / $cals['total_sorts']) * 100;

  # Joins
  $cals['joins_without_indexes'] = $stat['Select_range_check'] + $stat['Select_full_join'];

  # Temporary tables
  if ($stat['Created_tmp_tables'] > 0 && $stat['Created_tmp_disk_tables'] > 0) 
    $cals['pct_temp_disk'] = ($stat['Created_tmp_disk_tables'] / ($stat['Created_tmp_tables'] + $stat['Created_tmp_disk_tables'])) * 100;

  # Thread cache
  $cals['thread_cache_hit_rate'] = 100 - (($stat['Threads_created'] / $stat['Connections']) * 100);

  return $cals;
}


function get_mysqlinfo() {
  $ret=array();
  $res=SQL_query("SHOW /*!50000 GLOBAL */ VARIABLES;");
  if ($res) {
    while ($row=SQL_fetch_row($res))
      $ret['vars'][$row[0]]=$row[1];
    SQL_free_result($res);
  }
  $res=SQL_query("SHOW /*!50000 GLOBAL */ STATUS;");
  if ($res) {
    while ($row=SQL_fetch_row($res))
      $ret['stat'][$row[0]]=$row[1];
    SQL_free_result($res);
  }
  if (isset($ret['vars']['version'])) {
    list($ret['vars']['version_major'],$ret['vars']['version_minor'],$rest)=explode('.',$ret['vars']['version']);
  }
  return $ret;
}

function get_meminfo() {
  $ret=array();

  $f=@fopen("/proc/meminfo","r");
  if ($f!==FALSE) {
    while (!feof($f)) {
      $str=fgets($f,1024);
      if (preg_match("#^MemTotal#i",$str))
        $ret['MemTotal']=preg_replace('#[^0-9]#','',$str)*1024;
      if (preg_match("#^MemFree#i",$str))
        $ret['MemFree']=preg_replace('#[^0-9]#','',$str)*1024;
      if (preg_match("#^SwapTotal#i",$str))
        $ret['SwapTotal']=preg_replace('#[^0-9]#','',$str)*1024;
      if (preg_match("#^SwapFree#i",$str))
        $ret['SwapFree']=preg_replace('#[^0-9]#','',$str)*1024;
    }
    fclose($f);
    return $ret;
  }

  $output=array();
  @exec("free -b",$output);
  if (count($output)) {
    foreach($output as $str) {
      $f=explode(" ",preg_replace('#[^0-9 ]#','',$str));
      $g=array();
      while (list($key,$val) = each ($f)) if ($val!="") $g[]=$val;
      if (preg_match("#^Mem#i",$str)) {
        $ret['MemTotal']=$g[0];
        $ret['MemFree']=$g[2];
      }
      if (preg_match("#^Swap#i",$str)) {
        $ret['SwapTotal']=$g[0];
        $ret['SwapFree']=$g[2];
      }
    }
    return $ret;
  }

  $f=@fopen("/usr/share/netflow/web/meminfo","r");
  if ($f!==FALSE) {
    while (!feof($f)) {
      $str=fgets($f,1024);
      if (preg_match("#^MemTotal#i",$str))
        $ret['MemTotal']=preg_replace('#[^0-9]#','',$str)*1024;
      if (preg_match("#^MemFree#i",$str))
        $ret['MemFree']=preg_replace('#[^0-9]#','',$str)*1024;
      if (preg_match("#^SwapTotal#i",$str))
        $ret['SwapTotal']=preg_replace('#[^0-9]#','',$str)*1024;
      if (preg_match("#^SwapFree#i",$str))
        $ret['SwapFree']=preg_replace('#[^0-9]#','',$str)*1024;
    }
    fclose($f);
    return $ret;
  }
  return $ret;
}

?>